<!-- Contributed by David Bau, in the public domain -->

<template>
<div class="vectorlist">
  <div v-for="(vector, index) in vectors" class="vector">
    <input v-model="vector.text">
    <button @click="selectVector(index)">&rarr;</button>
    <button @click="deleteVector(index)">x</button>
  </div>
  <div class="operation">
  <button @click="saveVector()">Save current sample</button>
  </div>
  <!-- TODO: Change this button to do something interesting -->
  <div class="operation">
  <button @click="applyVectorMath()">Apply vector math</button>
  </div>
  <div class="operation">
  <button @click="applyBoldingVector()">Apply Bolding Vector</button>
  </div>
  <div class="operation">
  <button @click="applyCursiveVector()">Apply Cursive Vector</button>
  </div>
  <div class="operation">
  <button @click="applyAvgBoldingVector()">Apply Avg Bolding Vector</button>
  </div>
  <!-- TODO: Add the KNN font ID button below -->
  <div v-for="(vector, index) in vectors" class="vector">
  <button @click="getKNN(index)">Get KNN of Current Sample</button>
  </div>
  <div v-for="(vector, index) in vectors" class="vector">
  <button @click="getAvgKNN()">Get KNN of Avg Font</button>
  </div>
</div>
</template>

<script>
import {Array1D, ENV} from 'deeplearn';

const math = ENV.math;

//This json file includes all of the Font IDs in our database and their 40-dimensional logits vector.
var json = require('../embeddings.json');

export default {
  props: {
    selectedSample: { },
    model: { },
    vectors: { type: Array, default: () => [ { text: "0" } ] }
  },
  methods: {
    saveVector() {
      this.selectedSample.data().then(x =>
         this.vectors.push({ text: Array.prototype.slice.call(x).join(',') })
      );
    },
    deleteVector(index) {
      this.vectors.splice(index, 1);
    },
    selectVector(index) {
      this.$emit("select", { selectedSample: this.model.fixdim(
           Array1D.new(this.vectors[index].text.split(',').map(parseFloat)))});
    },
    // TODO: Add useful vector space operations here -->
    applyVectorMath() {
      this.$emit("select", { selectedSample:
           math.add(this.selectedSample, this.model.fixdim(
               Array1D.new([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]))) } )
    },

    // TODO: Apply Bolding Feature -->
    applyBoldingVector() {
      // average out the 10 fonts w/ bolded feature to create bolding vector
      var bolded_font_list = [json [3377], json [39504], json[28376], json[8551], json[18308], json[10181], json[10537], json[53974], json[39744], json[54942]];

      var bolding_vector = new Array(40).fill(0);
      for(var fontId=0; fontId<bolded_font_list.length; fontId++){
        var vector = bolded_font_list[fontId];
        for(var index=0; index<bolding_vector.length; index++){
          bolding_vector[index] += vector[index];
        }
      }
      for(var index=0; index<bolding_vector.length; index++){
        bolding_vector[index] = bolding_vector[index]/40;
      }

      console.log(bolding_vector);

      this.$emit("select", { selectedSample:
           math.add(this.selectedSample, this.model.fixdim(
               Array1D.new(bolding_vector))) } )
    },

    // TODO: Apply Cursive Feature -->
    applyCursiveVector() {
      // average out the 10 fonts w/ bolded feature to create bolding vector
      var cursive_font_list = [json [54299], json [28716], json[3987], json[5908], json[46140], json[44839], json[19331], json[6989], json[16715], json[25283]];

      var cursive_vector = new Array(40).fill(0);
      for(var fontId=0; fontId<cursive_font_list.length; fontId++){
        var vector = cursive_font_list[fontId];
        for(var index=0; index<cursive_vector.length; index++){
          cursive_vector[index] += vector[index];
        }
      }
      for(var index=0; index<cursive_vector.length; index++){
        cursive_vector[index] = cursive_vector[index]/40;
      }

      console.log(cursive_vector);

      this.$emit("select", { selectedSample:
           math.add(this.selectedSample, this.model.fixdim(
               Array1D.new(cursive_vector))) } )
    },

   // TODO: Apply Bolding Feature -->
    applyAvgBoldingVector() {
      // average out the 10 fonts w/ bolded feature to create bolding vector
      var bolded_font_list = [json [3377], json [39504], json[28376], json[8551], json[18308], json[10181], json[10537], json[53974], json[39744], json[54942]];
      var unbolded_font_list = [json [4315], json [5908], json[31361], json[7693], json[16067], json[7090], json[19297], json[52894], json[26294], json[29783]];
      var avg_bolded_font_list = new Array(40).fill(0);

      for(var fontId=0; fontId<bolded_font_list.length; fontId++){
        var bolded_vector = bolded_font_list[fontId];
        var unbolded_vector = unbolded_font_list[fontId];
        var avg_bolded_vector = new Array(40).fill(0);
        for(var index=0; index<bolded_vector.length; index++){
          avg_bolded_vector[index] = bolded_vector[index] - unbolded_vector[index];
        }
        avg_bolded_font_list[fontId] = avg_bolded_vector;
      }
      console.log(avg_bolded_font_list);
      
      var bolding_vector = new Array(40).fill(0);
      for(var fontId=0; fontId<avg_bolded_font_list.length; fontId++){
        var vector = avg_bolded_font_list[fontId];
        for(var index=0; index<bolding_vector.length; index++){
          bolding_vector[index] += vector[index];
        }
      }
      for(var index=0; index<bolding_vector.length; index++){
        bolding_vector[index] = bolding_vector[index]/40;
      }

      console.log(bolding_vector);

      this.$emit("select", { selectedSample:
           math.add(this.selectedSample, this.model.fixdim(
               Array1D.new(bolding_vector))) } )
    },
    //TODO: Implement getKNN to output the font ID of the nearest neighbor
    getKNN(button_input) {
      var min_similarity = Infinity;
      var min_fontId = 0;
      var input_index = button_input;

      var currentVector = this.vectors[input_index].text.split(',').map(parseFloat);

      // iterate through each of the fonts
      for(var fontId=0; fontId<json.length; fontId++){
        var otherVector = json[fontId];
        var norm_currentVector = 0;
        var norm_otherVector = 0;
        var dotproduct=0;

        // get dot product between selected fontId and current iterated fontId
        for(var index=0; index<otherVector.length; index++){
          dotproduct += (currentVector[index] * otherVector[index]);
          norm_currentVector += (currentVector[index]*currentVector[index]);
          norm_otherVector += (otherVector[index]*otherVector[index]);
        }
        norm_currentVector = Math.sqrt(norm_currentVector);
        norm_otherVector = Math.sqrt(norm_otherVector);
        var similarity = Math.abs(dotproduct/(norm_currentVector*norm_otherVector));

        // update new minimum similarity
        if(similarity < min_similarity){
          console.log('NEW MIN SIMILARITY');
          min_similarity = similarity;
          min_fontId = fontId;
        }
        console.log(fontId + ': ' + similarity);
      }
      console.log('most similar fontId: ' + min_fontId + ', min_similarity: ' + min_similarity);
    },

    //TODO: Implement getKNN to output the font ID of the nearest neighbor to the average font
    getAvgKNN() {
      // calculate average font vector
      var average_vector = new Array(40).fill(0);
      for(var fontId=0; fontId<json.length; fontId++){
        var vector = json[fontId];
        for(var index=0; index<average_vector.length; index++){
          average_vector[index] += vector[index];
        }
      }
      for(var index=0; index<average_vector.length; index++){
        average_vector[index] = average_vector[index]/40;
      }

      var min_similarity = Infinity;
      var min_fontId = 0;

      var currentVector = average_vector;

      // iterate through each of the fonts
      for(var fontId=0; fontId<json.length; fontId++){
        var otherVector = json[fontId];
        var norm_currentVector = 0;
        var norm_otherVector = 0;
        var dotproduct=0;

        // get dot product between selected fontId and current iterated fontId
        for(var index=0; index<otherVector.length; index++){
          dotproduct += (currentVector[index] * otherVector[index]);
          norm_currentVector += (currentVector[index]*currentVector[index]);
          norm_otherVector += (otherVector[index]*otherVector[index]);
        }
        norm_currentVector = Math.sqrt(norm_currentVector);
        norm_otherVector = Math.sqrt(norm_otherVector);
        var similarity = Math.abs(dotproduct/(norm_currentVector*norm_otherVector));

        // update new minimum similarity
        if(similarity < min_similarity){
          console.log('NEW MIN SIMILARITY');
          min_similarity = similarity;
          min_fontId = fontId;
        }
        console.log(fontId + ': ' + similarity);
      }
      console.log('most similar fontId: ' + min_fontId + ', min_similarity: ' + min_similarity);
    }


  },
  watch: {
    model: function(val) {
      for (let i = 0; i < this.vectors.length; ++i) {
        let arr = this.vectors[i].text.split(',');
        if (arr.length > this.model.dimensions) {
            arr = arr.slice(0, this.model.dimensions);
        }
        while (arr.length < this.model.dimensions) {
            arr.push('0');
        }
        this.vectors[i].text = arr.join(',');
      }
    }
  },
}
</script>

<style scoped>
.vector, .operation {
  border-top: 1px solid rgba(0, 0, 0, 0.1);
  white-space: nowrap;
}

</style>
